/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.source.service;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.DataSource;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spagobi.kpi.utils.AbstractConfigurableListModule;

import java.util.HashMap;
import java.util.List;

import org.apache.log4j.Logger;

public class ListEDataSourceModule extends AbstractConfigurableListModule {
	private static transient Logger logger = Logger
			.getLogger(ListEDataSourceModule.class);

	@Override
	public void service(SourceBean request, SourceBean response)
			throws Exception {

		super.service(request, response);

		String sourceTypeId = (String) request.getAttribute("TYPE_ID");

		HashMap parametersMap = (HashMap) response
				.getAttribute("PARAMETERS_MAP");
		if (parametersMap == null) {
			parametersMap = new HashMap();
			parametersMap.put("TYPE_ID", sourceTypeId);
			response.setAttribute("PARAMETERS_MAP", parametersMap);
		} else {
			parametersMap.put("TYPE_ID", sourceTypeId);
			response.updAttribute("PARAMETERS_MAP", parametersMap);
		}
	}

	@Override
	protected List getObjectList(SourceBean request) {
		String fieldOrder = (String) request.getAttribute("FIELD_ORDER");
		String typeOrder = (String) request.getAttribute("TYPE_ORDER");
		List result = null;
		try {
			String sourceTypeId = (String) request.getAttribute("TYPE_ID");
			if (sourceTypeId != null && !sourceTypeId.trim().equals("")) {
				IES4QFKDAO iEDataSourceDAO = (IES4QFKDAO) DAOFactory
						.getDAO("EDataSourceDAO");
				result = iEDataSourceDAO.loadObjectList(Integer
						.parseInt(sourceTypeId), fieldOrder, typeOrder);
			}
		} catch (EMFUserError e) {
			logger.error(e);
		}
		return result;
	}

	@Override
	protected void setRowAttribute(SourceBean rowSB, Object obj)
			throws SourceBeanException {
		DataSource aDataSource = (DataSource) obj;
		rowSB.setAttribute("ID", aDataSource.getId());
		if (aDataSource.getSourceType() != null){
			rowSB.setAttribute("NAME_TYPE", aDataSource.getSourceType().getName());
		}
		rowSB.setAttribute("NAME", aDataSource.getName());
	}

	@Override
	public boolean delete(SourceBean request, SourceBean response) {
		boolean toReturn = false;
		String id = (String) request.getAttribute("ID");
		try {
			IES4QFKDAO dataSourceDAO = (IES4QFKDAO) DAOFactory.getDAO("EDataSourceDAO");
			toReturn = dataSourceDAO.deleteObject(Integer.parseInt(id));
			toReturn = true;
		} catch (NumberFormatException e) {
			EMFErrorHandler engErrorHandler = getErrorHandler();
			engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.WARNING,
					"10012", "component_spago4q_messages"));
		} catch (EMFUserError e) {
			EMFErrorHandler engErrorHandler = getErrorHandler();
			engErrorHandler.addError(e);
		}

		return toReturn;
	}

}
